@@ -1,6 +1,9 @@
 /*
  * dbus.h -- D-BUS interface
- * Copyright (C) 2007 Alberto Garcia <agarcia@igalia.com>
+ *
+ * Copyright (C) 2007-2008 Igalia, S.L.
+ * Authors: Mario Sanchez Prada <msanchez@igalia.com>
+ *          Alberto Garcia <agarcia@igalia.com>
  *
  * This file is part of Vagalume and is published under the GNU GPLv3
  * See the README file for more details.
@@ -11,6 +14,7 @@
 
 #include "globaldefs.h"
 #include "playlist.h"
+#include "controller.h"
 
 #define APP_DBUS_SERVICE "com.igalia." APP_NAME_LC
 #define APP_DBUS_OBJECT "/com/igalia/" APP_NAME_LC
@@ -62,24 +66,22 @@ typedef enum {
         DBUS_INIT_ALREADY_RUNNING
 } DbusInitReturnCode;
 
-#ifdef HAVE_DBUS_SUPPORT
+DbusInitReturnCode
+lastfm_dbus_init                        (VglController *controller);
 
-DbusInitReturnCode lastfm_dbus_init(void);
-void lastfm_dbus_close(void);
-void lastfm_dbus_notify_playback(LastfmTrack *track);
-void lastfm_dbus_notify_started(void);
-void lastfm_dbus_notify_closing(void);
-void lastfm_dbus_play_radio_url(const char *url);
+void
+lastfm_dbus_close                       (void);
 
-#else
+void
+lastfm_dbus_notify_playback             (LastfmTrack *track);
 
-DbusInitReturnCode lastfm_dbus_init(void) { return DBUS_INIT_OK; }
-void lastfm_dbus_close(void) { }
-void lastfm_dbus_notify_playback(LastfmTrack *track) { }
-void lastfm_dbus_notify_started(void) { }
-void lastfm_dbus_notify_closing(void) { }
-void lastfm_dbus_play_radio_url(const char *url) { }
+void
+lastfm_dbus_notify_started              (void);
 
-#endif /* HAVE_DBUS_SUPPORT */
+void
+lastfm_dbus_notify_closing              (void);
+
+void
+lastfm_dbus_play_radio_url              (const char *url);
 
 #endif
