@@ -1,7 +1,7 @@
 /*
  * globaldefs.h -- Some global definitions
  *
- * Copyright (C) 2007-2008 Igalia, S.L.
+ * Copyright (C) 2007-2010 Igalia, S.L.
  * Authors: Alberto Garcia <agarcia@igalia.com>
  *
  * This file is part of Vagalume and is published under the GNU GPLv3
@@ -33,4 +33,8 @@
 
 #define THEME_ICONS_DIR APP_DATA_DIR "/icons"
 
+#ifdef MAEMO5
+#    define FINGER_SIZE (HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH)
+#endif
+
 #endif
