@@ -1,6 +1,9 @@
 /*
  * vgl-sb-plugin.h -- Status bar plugin for Vagalume
- * Copyright (C) 2008 Mario Sanchez Prada <msanchez@igalia.com>
+ *
+ * Copyright (C) 2007-2008 Igalia, S.L.
+ * Authors: Alberto Garcia <agarcia@igalia.com>
+ *          Mario Sanchez Prada <msanchez@igalia.com>
  *
  * This file is part of Vagalume and is published under the GNU GPLv3
  * See the README file for more details.
@@ -18,12 +21,14 @@
 #define VGL_IS_SB_PLUGIN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  VGL_SB_PLUGIN_TYPE))
 #define VGL_SB_PLUGIN_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  VGL_SB_PLUGIN_TYPE, VglSbPluginClass))
 
-typedef struct _VglSbPlugin      VglSbPlugin;
-typedef struct _VglSbPluginClass VglSbPluginClass;
+typedef struct _VglSbPlugin        VglSbPlugin;
+typedef struct _VglSbPluginClass   VglSbPluginClass;
+typedef struct _VglSbPluginPrivate VglSbPluginPrivate;
 
 struct _VglSbPlugin
 {
   StatusbarItem parent;
+  VglSbPluginPrivate *priv;
 };
 
 struct _VglSbPluginClass
@@ -31,6 +36,7 @@ struct _VglSbPluginClass
   StatusbarItemClass parent_class;
 };
 
-GType vgl_sb_plugin_get_type(void);
+GType
+vgl_sb_plugin_get_type                  (void);
 
 #endif /* !VGL_SB_PLUGIN_H */
