@@ -1,6 +1,9 @@
 /*
  * vgl-tray-icon.c -- Freedesktop tray icon
- * Copyright (C) 2008 Mario Sanchez Prada <msanchez@igalia.com>
+ *
+ * Copyright (C) 2007-2009 Igalia, S.L.
+ * Authors: Mario Sanchez Prada <msanchez@igalia.com>
+ *          Alberto Garcia <agarcia@igalia.com>
  *
  * This file is part of Vagalume and is published under the GNU GPLv3.
  * See the README file for more details.
@@ -12,10 +15,10 @@
 #include "vgl-tray-icon.h"
 #include "globaldefs.h"
 
-#include "controller.h"
-#include "playlist.h"
-#include "metadata.h"
 #include "util.h"
+#include "uimisc.h"
+#include "scrobbler.h"
+#include "compat.h"
 
 #define TOOLTIP_DEFAULT_STRING _(" Stopped ")
 #define TOOLTIP_FORMAT_STRING  _(" Now playing: \n %s \n   by  %s ")
@@ -23,41 +26,30 @@
 #define NOTIFICATION_BODY_NO_ALBUM _(" by <i>%s</i>")
 #define NOTIFICATION_BODY_WITH_ALBUM _(" by <i>%s</i>\n from <i>%s</i>")
 
-#define SHOW_APP_ITEM_STRING _("Show main window")
-#define SETTINGS_ITEM_STRING _("Settings...")
-#define RECOMMEND_ITEM_STRING _("Recommend...")
-#define TAG_ITEM_STRING _("Tags...")
-#define ADD_TO_PLS_ITEM_STRING _("Add to playlist")
-#define LOVE_ITEM_STRING _("Love this track")
-#define BAN_ITEM_STRING _("Ban this track")
-#define PLAY_ITEM_STRING _("Play")
-#define STOP_ITEM_STRING _("Stop")
-#define SKIP_ITEM_STRING _("Skip")
-#define CLOSE_APP_ITEM_STRING _("Close Vagalume")
-
-#define MENU_ITEM_ICON_SIZE 18
 #define NOTIFICATION_ICON_SIZE 48
 
-#define SHOW_APP_ITEM_ICON_NAME "view-restore"
-#define SETTINGS_ITEM_ICON_NAME "preferences-other"
-#define RECOMMEND_ITEM_ICON_NAME "mail-message-new"
-#define TAG_ITEM_ICON_NAME "accessories-text-editor"
-#define ADD_TO_PLS_ITEM_ICON_NAME "list-add"
-#define LOVE_ITEM_ICON_NAME "emblem-favorite"
-#define BAN_ITEM_ICON_NAME "process-stop"
-#define PLAY_ITEM_ICON_NAME "media-playback-start"
-#define STOP_ITEM_ICON_NAME "media-playback-stop"
-#define SKIP_ITEM_ICON_NAME "media-skip-forward"
-#define CLOSE_APP_ITEM_ICON_NAME "window-close"
-
 #define VGL_TRAY_ICON_GET_PRIVATE(object)      \
         (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
                                       VGL_TRAY_ICON_TYPE, VglTrayIconPrivate))
 
 G_DEFINE_TYPE (VglTrayIcon, vgl_tray_icon, G_TYPE_OBJECT);
 
+enum {
+        SETTINGS_ITEM,
+        ABOUT_ITEM,
+        RECOMMEND_ITEM,
+        TAG_ITEM,
+        ADD_TO_PLS_ITEM,
+        LOVE_ITEM,
+        BAN_ITEM,
+        PLAY_ITEM,
+        STOP_ITEM,
+        NEXT_ITEM,
+        QUIT_ITEM,
+        N_MENU_ITEMS
+};
+
 /* Private struct */
-typedef struct _VglTrayIconPrivate VglTrayIconPrivate;
 struct _VglTrayIconPrivate
 {
         GtkStatusIcon *tray_icon;
@@ -66,34 +58,14 @@ struct _VglTrayIconPrivate
 
         GtkWidget *ctxt_menu;
 
-        GtkWidget *show_app_item;
-        GtkWidget *settings_item;
-        GtkWidget *recommend_item;
-        GtkWidget *tag_item;
-        GtkWidget *add_to_pls_item;
-        GtkWidget *love_item;
-        GtkWidget *ban_item;
-        GtkWidget *play_item;
-        GtkWidget *stop_item;
-        GtkWidget *next_item;
-        GtkWidget *close_vagalume_item;
+        GtkWidget *menu_item[N_MENU_ITEMS];
+        gulong menu_handler_id[N_MENU_ITEMS];
 
         gboolean show_notifications;
         NotifyNotification *notification;
 
-        gint tray_icon_clicked_handler_id;
-        gint tray_icon_popup_menu_handler_id;
-        gint show_app_item_handler_id;
-        gint settings_item_handler_id;
-        gint recommend_item_handler_id;
-        gint tag_item_handler_id;
-        gint add_to_pls_item_handler_id;
-        gint love_item_handler_id;
-        gint ban_item_handler_id;
-        gint play_item_handler_id;
-        gint stop_item_handler_id;
-        gint next_item_handler_id;
-        gint close_vagalume_item_handler_id;
+        gulong tray_icon_clicked_handler_id;
+        gulong tray_icon_popup_menu_handler_id;
 };
 
 /* Private */
@@ -120,12 +92,15 @@ static void tray_icon_popup_menu (GtkStatusIcon *status_icon, guint button,
 
 static void ctxt_menu_item_activated (GtkWidget *item, gpointer data);
 
+static void vgl_tray_icon_notify_playback (VglTrayIcon *vti,
+                                           LastfmTrack *track);
+
 /* Private */
 
 /* Initialization/destruction functions */
 
 static void
-vgl_tray_icon_class_init(VglTrayIconClass *klass)
+vgl_tray_icon_class_init                (VglTrayIconClass *klass)
 {
         GObjectClass *obj_class = G_OBJECT_CLASS(klass);
 
@@ -134,26 +109,21 @@ vgl_tray_icon_class_init(VglTrayIconClass *klass)
 }
 
 static void
-vgl_tray_icon_init (VglTrayIcon *vti)
+vgl_tray_icon_init                      (VglTrayIcon *vti)
 {
+        int i;
         VglTrayIconPrivate *priv = VGL_TRAY_ICON_GET_PRIVATE (vti);
 
         /* Init private attributes */
+        vti->priv = priv;
         priv->tray_icon = gtk_status_icon_new ();
 
         priv->now_playing = FALSE;
 
-        priv->show_app_item = NULL;
-        priv->settings_item = NULL;
-        priv->recommend_item = NULL;
-        priv->tag_item = NULL;
-        priv->add_to_pls_item = NULL;
-        priv->love_item = NULL;
-        priv->ban_item = NULL;
-        priv->play_item = NULL;
-        priv->stop_item = NULL;
-        priv->next_item = NULL;
-        priv->close_vagalume_item = NULL;
+        for (i = 0; i < N_MENU_ITEMS; i++) {
+                priv->menu_item[i] = NULL;
+                priv->menu_handler_id[i] = 0;
+        }
 
         priv->notification = NULL;
         priv->show_notifications = FALSE;
@@ -183,37 +153,20 @@ vgl_tray_icon_init (VglTrayIcon *vti)
 }
 
 static void
-vgl_tray_icon_finalize (GObject* object)
+vgl_tray_icon_finalize                  (GObject* object)
 {
+        int i;
         VglTrayIcon *vti = VGL_TRAY_ICON (object);
-        VglTrayIconPrivate *priv = VGL_TRAY_ICON_GET_PRIVATE (vti);
+        VglTrayIconPrivate *priv = vti->priv;
 
         /* Cleanup libnotify */
         cleanup_libnotify (vti);
 
         /* Disconnect handlers */
-        g_signal_handler_disconnect (priv->show_app_item,
-                                     priv->show_app_item_handler_id);
-        g_signal_handler_disconnect (priv->settings_item,
-                                     priv->settings_item_handler_id);
-        g_signal_handler_disconnect (priv->recommend_item,
-                                     priv->recommend_item_handler_id);
-        g_signal_handler_disconnect (priv->tag_item,
-                                     priv->tag_item_handler_id);
-        g_signal_handler_disconnect (priv->add_to_pls_item,
-                                     priv->add_to_pls_item_handler_id);
-        g_signal_handler_disconnect (priv->love_item,
-                                     priv->love_item_handler_id);
-        g_signal_handler_disconnect (priv->ban_item,
-                                     priv->ban_item_handler_id);
-        g_signal_handler_disconnect (priv->play_item,
-                                     priv->play_item_handler_id);
-        g_signal_handler_disconnect (priv->stop_item,
-                                     priv->stop_item_handler_id);
-        g_signal_handler_disconnect (priv->next_item,
-                                     priv->next_item_handler_id);
-        g_signal_handler_disconnect (priv->close_vagalume_item,
-                                     priv->close_vagalume_item_handler_id);
+        for (i = 0; i < N_MENU_ITEMS; i++) {
+                g_signal_handler_disconnect (priv->menu_item[i],
+                                             priv->menu_handler_id[i]);
+        }
         g_signal_handler_disconnect (priv->tray_icon,
                                      priv->tray_icon_clicked_handler_id);
         g_signal_handler_disconnect (priv->tray_icon,
@@ -234,7 +187,7 @@ vgl_tray_icon_finalize (GObject* object)
 
 /* Libnotify functions */
 static void
-setup_libnotify  (VglTrayIcon *vti)
+setup_libnotify                         (VglTrayIcon *vti)
 {
         if (!notify_init (APP_NAME)) {
                 g_debug ("[TRAY ICON] :: Error initializing libnotify");
@@ -244,9 +197,9 @@ setup_libnotify  (VglTrayIcon *vti)
 }
 
 static void
-cleanup_libnotify  (VglTrayIcon *vti)
+cleanup_libnotify                       (VglTrayIcon *vti)
 {
-        VglTrayIconPrivate *priv = VGL_TRAY_ICON_GET_PRIVATE (vti);
+        VglTrayIconPrivate *priv = vti->priv;
 
         if (notify_is_initted ()) {
                 if (priv->notification) {
@@ -259,106 +212,95 @@ cleanup_libnotify  (VglTrayIcon *vti)
 
 /* Panel update functions */
 
-static GtkWidget *
-ctxt_menu_item_create (const gchar *icon_name, const gchar *label)
-{
-        g_return_val_if_fail (icon_name != NULL && label != NULL, NULL);
-
-        GtkWidget *item = gtk_image_menu_item_new_with_label (label);
-        GtkIconTheme *icon_theme = gtk_icon_theme_get_default();
-        GdkPixbuf *pixbuf = gtk_icon_theme_load_icon(icon_theme, icon_name,
-                                                     MENU_ITEM_ICON_SIZE, 0, NULL);
-
-        if (pixbuf != NULL) {
-                GtkWidget *image = gtk_image_new_from_pixbuf(pixbuf);
-                gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
-                                               image);
-                g_object_unref (pixbuf);
-        }
-
-        return item;
-}
-
 static void
-ctxt_menu_create (VglTrayIcon *vti)
+ctxt_menu_create                        (VglTrayIcon *vti)
 {
-        VglTrayIconPrivate *priv = VGL_TRAY_ICON_GET_PRIVATE (vti);
+        VglTrayIconPrivate *priv = vti->priv;
+        GtkMenuShell *ctxt_menu;
 
         /* Create ctxt_menu and ctxt_menu items */
         priv->ctxt_menu = gtk_menu_new ();
-        priv->show_app_item =
-                ctxt_menu_item_create (SHOW_APP_ITEM_ICON_NAME, SHOW_APP_ITEM_STRING);
-        priv->settings_item =
-                ctxt_menu_item_create (SETTINGS_ITEM_ICON_NAME, SETTINGS_ITEM_STRING);
-        priv->recommend_item =
-                ctxt_menu_item_create (RECOMMEND_ITEM_ICON_NAME, RECOMMEND_ITEM_STRING);
-        priv->tag_item =
-                ctxt_menu_item_create (TAG_ITEM_ICON_NAME, TAG_ITEM_STRING);
-        priv->add_to_pls_item =
-                ctxt_menu_item_create (ADD_TO_PLS_ITEM_ICON_NAME, ADD_TO_PLS_ITEM_STRING);
-        priv->love_item =
-                ctxt_menu_item_create (LOVE_ITEM_ICON_NAME, LOVE_ITEM_STRING);
-        priv->ban_item =
-                ctxt_menu_item_create (BAN_ITEM_ICON_NAME, BAN_ITEM_STRING);
-        priv->play_item =
-                ctxt_menu_item_create (PLAY_ITEM_ICON_NAME, PLAY_ITEM_STRING);
-        priv->stop_item =
-                ctxt_menu_item_create (STOP_ITEM_ICON_NAME, STOP_ITEM_STRING);
-        priv->next_item =
-                ctxt_menu_item_create (SKIP_ITEM_ICON_NAME, SKIP_ITEM_STRING);
-        priv->close_vagalume_item =
-                ctxt_menu_item_create (CLOSE_APP_ITEM_ICON_NAME, CLOSE_APP_ITEM_STRING);
+        ctxt_menu = GTK_MENU_SHELL (priv->ctxt_menu);
+
+        priv->menu_item[SETTINGS_ITEM] =
+                gtk_image_menu_item_new_from_stock (GTK_STOCK_PREFERENCES,
+                                                    NULL);
+        priv->menu_item[ABOUT_ITEM] =
+                gtk_image_menu_item_new_from_stock (GTK_STOCK_ABOUT, NULL);
+        priv->menu_item[RECOMMEND_ITEM] =
+                ui_menu_item_create_from_icon (RECOMMEND_ITEM_ICON_NAME,
+                                               RECOMMEND_ITEM_STRING);
+        priv->menu_item[TAG_ITEM] =
+                ui_menu_item_create_from_icon (TAG_ITEM_ICON_NAME,
+                                               TAG_ITEM_STRING);
+        priv->menu_item[ADD_TO_PLS_ITEM] =
+                ui_menu_item_create_from_icon (ADD_TO_PLS_ITEM_ICON_NAME,
+                                               ADD_TO_PLS_ITEM_STRING);
+        priv->menu_item[LOVE_ITEM] =
+                ui_menu_item_create_from_icon (LOVE_ITEM_ICON_NAME,
+                                               LOVE_ITEM_STRING);
+        priv->menu_item[BAN_ITEM] =
+                ui_menu_item_create_from_icon (BAN_ITEM_ICON_NAME,
+                                               BAN_ITEM_STRING);
+        priv->menu_item[PLAY_ITEM] =
+                gtk_image_menu_item_new_from_stock (GTK_STOCK_MEDIA_PLAY,NULL);
+        priv->menu_item[STOP_ITEM] =
+                gtk_image_menu_item_new_from_stock (GTK_STOCK_MEDIA_STOP,NULL);
+        priv->menu_item[NEXT_ITEM] =
+                gtk_image_menu_item_new_from_stock (GTK_STOCK_MEDIA_NEXT,NULL);
+        priv->menu_item[QUIT_ITEM] =
+                gtk_image_menu_item_new_from_stock (GTK_STOCK_QUIT, NULL);
 
         /* Add items to ctxt_menu */
-        gtk_menu_append (priv->ctxt_menu, priv->show_app_item);
-        gtk_menu_append (priv->ctxt_menu, priv->settings_item);
-        gtk_menu_append (priv->ctxt_menu, gtk_separator_menu_item_new ());
-        gtk_menu_append (priv->ctxt_menu, priv->recommend_item);
-        gtk_menu_append (priv->ctxt_menu, priv->tag_item);
-        gtk_menu_append (priv->ctxt_menu, priv->add_to_pls_item);
-        gtk_menu_append (priv->ctxt_menu, gtk_separator_menu_item_new ());
-        gtk_menu_append (priv->ctxt_menu, priv->love_item);
-        gtk_menu_append (priv->ctxt_menu, priv->ban_item);
-        gtk_menu_append (priv->ctxt_menu, gtk_separator_menu_item_new ());
-        gtk_menu_append (priv->ctxt_menu, priv->play_item);
-        gtk_menu_append (priv->ctxt_menu, priv->stop_item);
-        gtk_menu_append (priv->ctxt_menu, priv->next_item);
-        gtk_menu_append (priv->ctxt_menu, gtk_separator_menu_item_new ());
-        gtk_menu_append (priv->ctxt_menu, priv->close_vagalume_item);
+        gtk_menu_shell_append (ctxt_menu, priv->menu_item[RECOMMEND_ITEM]);
+        gtk_menu_shell_append (ctxt_menu, priv->menu_item[TAG_ITEM]);
+        gtk_menu_shell_append (ctxt_menu, priv->menu_item[ADD_TO_PLS_ITEM]);
+        gtk_menu_shell_append (ctxt_menu, gtk_separator_menu_item_new ());
+        gtk_menu_shell_append (ctxt_menu, priv->menu_item[LOVE_ITEM]);
+        gtk_menu_shell_append (ctxt_menu, priv->menu_item[BAN_ITEM]);
+        gtk_menu_shell_append (ctxt_menu, gtk_separator_menu_item_new ());
+        gtk_menu_shell_append (ctxt_menu, priv->menu_item[PLAY_ITEM]);
+        gtk_menu_shell_append (ctxt_menu, priv->menu_item[STOP_ITEM]);
+        gtk_menu_shell_append (ctxt_menu, priv->menu_item[NEXT_ITEM]);
+        gtk_menu_shell_append (ctxt_menu, gtk_separator_menu_item_new ());
+        gtk_menu_shell_append (ctxt_menu, priv->menu_item[SETTINGS_ITEM]);
+        gtk_menu_shell_append (ctxt_menu, priv->menu_item[ABOUT_ITEM]);
+        gtk_menu_shell_append (ctxt_menu, gtk_separator_menu_item_new ());
+        gtk_menu_shell_append (ctxt_menu, priv->menu_item[QUIT_ITEM]);
 
         /* Connect signals */
-        priv->show_app_item_handler_id =
-                g_signal_connect(priv->show_app_item, "activate",
+        priv->menu_handler_id[SETTINGS_ITEM] =
+                g_signal_connect(priv->menu_item[SETTINGS_ITEM], "activate",
                                  G_CALLBACK (ctxt_menu_item_activated), vti);
-        priv->settings_item_handler_id =
-                g_signal_connect(priv->settings_item, "activate",
+        priv->menu_handler_id[ABOUT_ITEM] =
+                g_signal_connect(priv->menu_item[ABOUT_ITEM], "activate",
                                  G_CALLBACK (ctxt_menu_item_activated), vti);
-        priv->recommend_item_handler_id =
-                g_signal_connect(priv->recommend_item, "activate",
+        priv->menu_handler_id[RECOMMEND_ITEM] =
+                g_signal_connect(priv->menu_item[RECOMMEND_ITEM], "activate",
                                  G_CALLBACK (ctxt_menu_item_activated), vti);
-        priv->tag_item_handler_id =
-                g_signal_connect(priv->tag_item, "activate",
+        priv->menu_handler_id[TAG_ITEM] =
+                g_signal_connect(priv->menu_item[TAG_ITEM], "activate",
                                  G_CALLBACK (ctxt_menu_item_activated), vti);
-        priv->add_to_pls_item_handler_id =
-                g_signal_connect(priv->add_to_pls_item, "activate",
+        priv->menu_handler_id[ADD_TO_PLS_ITEM] =
+                g_signal_connect(priv->menu_item[ADD_TO_PLS_ITEM], "activate",
                                  G_CALLBACK (ctxt_menu_item_activated), vti);
-        priv->love_item_handler_id =
-                g_signal_connect(priv->love_item, "activate",
+        priv->menu_handler_id[LOVE_ITEM] =
+                g_signal_connect(priv->menu_item[LOVE_ITEM], "activate",
                                  G_CALLBACK (ctxt_menu_item_activated), vti);
-        priv->ban_item_handler_id =
-                g_signal_connect(priv->ban_item, "activate",
+        priv->menu_handler_id[BAN_ITEM] =
+                g_signal_connect(priv->menu_item[BAN_ITEM], "activate",
                                  G_CALLBACK (ctxt_menu_item_activated), vti);
-        priv->play_item_handler_id =
-                g_signal_connect(priv->play_item, "activate",
+        priv->menu_handler_id[PLAY_ITEM] =
+                g_signal_connect(priv->menu_item[PLAY_ITEM], "activate",
                                  G_CALLBACK (ctxt_menu_item_activated), vti);
-        priv->stop_item_handler_id =
-                g_signal_connect(priv->stop_item, "activate",
+        priv->menu_handler_id[STOP_ITEM] =
+                g_signal_connect(priv->menu_item[STOP_ITEM], "activate",
                                  G_CALLBACK (ctxt_menu_item_activated), vti);
-        priv->next_item_handler_id =
-                g_signal_connect(priv->next_item, "activate",
+        priv->menu_handler_id[NEXT_ITEM] =
+                g_signal_connect(priv->menu_item[NEXT_ITEM], "activate",
                                  G_CALLBACK (ctxt_menu_item_activated), vti);
-        priv->close_vagalume_item_handler_id =
-                g_signal_connect(priv->close_vagalume_item, "activate",
+        priv->menu_handler_id[QUIT_ITEM] =
+                g_signal_connect(priv->menu_item[QUIT_ITEM], "activate",
                                  G_CALLBACK (ctxt_menu_item_activated), vti);
 
         /* Show widgets */
@@ -366,54 +308,48 @@ ctxt_menu_create (VglTrayIcon *vti)
 }
 
 static void
-ctxt_menu_update (VglTrayIcon *vti)
+ctxt_menu_update                        (VglTrayIcon *vti)
 {
-        VglTrayIconPrivate *priv = VGL_TRAY_ICON_GET_PRIVATE (vti);
+        VglTrayIconPrivate *priv = vti->priv;
+        gboolean np = priv->now_playing;
 
         /* Adjust sentitiveness for ctxt_menu items and show/hide buttons */
-        if (priv->now_playing) {
-                gtk_widget_set_sensitive (priv->recommend_item, TRUE);
-                gtk_widget_set_sensitive (priv->tag_item, TRUE);
-                gtk_widget_set_sensitive (priv->add_to_pls_item, TRUE);
-                gtk_widget_set_sensitive (priv->love_item, TRUE);
-                gtk_widget_set_sensitive (priv->ban_item, TRUE);
-                gtk_widget_set_sensitive (priv->play_item, FALSE);
-                gtk_widget_set_sensitive (priv->stop_item, TRUE);
-                gtk_widget_set_sensitive (priv->next_item, TRUE);
+        gtk_widget_set_sensitive (priv->menu_item[RECOMMEND_ITEM], np);
+        gtk_widget_set_sensitive (priv->menu_item[TAG_ITEM], np);
+        gtk_widget_set_sensitive (priv->menu_item[ADD_TO_PLS_ITEM], np);
+        gtk_widget_set_sensitive (priv->menu_item[LOVE_ITEM], np);
+        gtk_widget_set_sensitive (priv->menu_item[BAN_ITEM], np);
+        gtk_widget_set_sensitive (priv->menu_item[PLAY_ITEM], !np);
+        gtk_widget_set_sensitive (priv->menu_item[STOP_ITEM], np);
+        gtk_widget_set_sensitive (priv->menu_item[NEXT_ITEM], np);
 
-                gtk_widget_hide (priv->play_item);
-                gtk_widget_show (priv->stop_item);
+        if (np) {
+                gtk_widget_hide (priv->menu_item[PLAY_ITEM]);
+                gtk_widget_show (priv->menu_item[STOP_ITEM]);
         } else {
-                gtk_widget_set_sensitive (priv->recommend_item, FALSE);
-                gtk_widget_set_sensitive (priv->tag_item, FALSE);
-                gtk_widget_set_sensitive (priv->add_to_pls_item, FALSE);
-                gtk_widget_set_sensitive (priv->love_item, FALSE);
-                gtk_widget_set_sensitive (priv->ban_item, FALSE);
-                gtk_widget_set_sensitive (priv->play_item, TRUE);
-                gtk_widget_set_sensitive (priv->stop_item, FALSE);
-                gtk_widget_set_sensitive (priv->next_item, FALSE);
-
-                gtk_widget_show (priv->play_item);
-                gtk_widget_hide (priv->stop_item);
+                gtk_widget_show (priv->menu_item[PLAY_ITEM]);
+                gtk_widget_hide (priv->menu_item[STOP_ITEM]);
         }
-        gtk_widget_show (priv->close_vagalume_item);
 }
 
 
 /* Signals handlers */
 
 static void
-tray_icon_clicked (GtkStatusIcon *status_icon, gpointer data)
+tray_icon_clicked                       (GtkStatusIcon *status_icon,
+                                         gpointer       data)
 {
         controller_toggle_mainwin_visibility ();
 }
 
 static void
-tray_icon_popup_menu (GtkStatusIcon *status_icon, guint button,
-                      guint activate_time, gpointer data)
+tray_icon_popup_menu                    (GtkStatusIcon *status_icon,
+                                         guint          button,
+                                         guint          activate_time,
+                                         gpointer       data)
 {
         VglTrayIcon *vti = VGL_TRAY_ICON (data);
-        VglTrayIconPrivate *priv = VGL_TRAY_ICON_GET_PRIVATE (vti);
+        VglTrayIconPrivate *priv = vti->priv;
 
         gtk_menu_popup (GTK_MENU (priv->ctxt_menu),
                         NULL, NULL,
@@ -424,49 +360,92 @@ tray_icon_popup_menu (GtkStatusIcon *status_icon, guint button,
 }
 
 static void
-ctxt_menu_item_activated (GtkWidget *item, gpointer data)
+ctxt_menu_item_activated                (GtkWidget *item,
+                                         gpointer   data)
 {
         VglTrayIcon *vti = VGL_TRAY_ICON (data);
-        VglTrayIconPrivate *priv = VGL_TRAY_ICON_GET_PRIVATE (vti);
+        VglTrayIconPrivate *priv = vti->priv;
 
-        if (item == priv->show_app_item) {
-                controller_show_mainwin(TRUE);
-        } else if (item == priv->settings_item) {
+        if (item == priv->menu_item[SETTINGS_ITEM]) {
                 controller_open_usercfg();
-        } else if (item == priv->recommend_item) {
+        } else if (item == priv->menu_item[ABOUT_ITEM]) {
+                controller_show_about();
+        } else if (item == priv->menu_item[RECOMMEND_ITEM]) {
                 controller_recomm_track();
-        } else if (item == priv->tag_item) {
+        } else if (item == priv->menu_item[TAG_ITEM]) {
                 controller_tag_track();
-        } else if (item == priv->add_to_pls_item) {
+        } else if (item == priv->menu_item[ADD_TO_PLS_ITEM]) {
                 controller_add_to_playlist();
-        } else if (item == priv->love_item) {
+        } else if (item == priv->menu_item[LOVE_ITEM]) {
                 controller_love_track (TRUE);
-        } else if (item == priv->ban_item) {
+        } else if (item == priv->menu_item[BAN_ITEM]) {
                 controller_ban_track (TRUE);
-        } else if (item == priv->play_item) {
+        } else if (item == priv->menu_item[PLAY_ITEM]) {
                 controller_start_playing ();
-        } else if (item == priv->stop_item) {
+        } else if (item == priv->menu_item[STOP_ITEM]) {
                 controller_stop_playing ();
-        } else if (item == priv->next_item) {
+        } else if (item == priv->menu_item[NEXT_ITEM]) {
                 controller_skip_track ();
-        } else if (item == priv->close_vagalume_item) {
+        } else if (item == priv->menu_item[QUIT_ITEM]) {
                 controller_quit_app ();
         } else {
                 g_warning ("[TRAY ICON] :: Unknown action");
         }
 }
 
+static void
+usercfg_changed_cb                      (VglController *ctrl,
+                                         VglUserCfg    *cfg,
+                                         VglTrayIcon   *icon)
+{
+        icon->priv->show_notifications = cfg->show_notifications;
+}
+
+static void
+track_stopped_cb                        (VglController *ctrl,
+                                         RspRating      rating,
+                                         VglTrayIcon   *icon)
+{
+        vgl_tray_icon_notify_playback (icon, NULL);
+}
+
+static void
+track_started_cb                        (VglController *ctrl,
+                                         LastfmTrack   *track,
+                                         VglTrayIcon   *icon)
+{
+        vgl_tray_icon_notify_playback (icon, track);
+}
+
+static void
+controller_destroyed_cb                 (gpointer  data,
+                                         GObject  *controller)
+{
+        VglTrayIcon *icon = VGL_TRAY_ICON (data);
+        g_object_unref (icon);
+}
 
 /* Public */
 
 VglTrayIcon *
-vgl_tray_icon_create (void)
+vgl_tray_icon_create                    (VglController *controller)
 {
-        return g_object_new(VGL_TRAY_ICON_TYPE, NULL);
+        VglTrayIcon *icon;
+        g_return_val_if_fail (VGL_IS_CONTROLLER (controller), NULL);
+        icon = g_object_new (VGL_TRAY_ICON_TYPE, NULL);
+        g_signal_connect (controller, "usercfg-changed",
+                          G_CALLBACK (usercfg_changed_cb), icon);
+        g_signal_connect (controller, "track-stopped",
+                          G_CALLBACK (track_stopped_cb), icon);
+        g_signal_connect (controller, "track-started",
+                          G_CALLBACK (track_started_cb), icon);
+        g_object_weak_ref (G_OBJECT (controller),
+                           controller_destroyed_cb, icon);
+        return icon;
 }
 
 static GdkPixbuf *
-get_default_album_cover_icon (void)
+get_default_album_cover_icon            (void)
 {
         static GdkPixbuf *pixbuf = NULL;
 
@@ -478,47 +457,28 @@ get_default_album_cover_icon (void)
         return g_object_ref (pixbuf);
 }
 
-static GdkPixbuf *
-get_album_cover_icon (LastfmTrack *track)
-{
-        g_return_val_if_fail(track != NULL && track->image_url != NULL, NULL);
-        GdkPixbuf *pixbuf = NULL;
-
-        /* This can take some time, so release the GDK lock */
-        gdk_threads_leave();
-        lastfm_get_track_cover_image(track);
-        gdk_threads_enter();
-
-        if (track->image_data != NULL) {
-                pixbuf = get_pixbuf_from_image(track->image_data,
-                                               track->image_data_size,
-                                               NOTIFICATION_ICON_SIZE);
-        } else {
-                g_debug("Error getting cover image");
-        }
-
-        return pixbuf;
-}
-
 typedef struct {
         LastfmTrack *track;
         VglTrayIcon *vti;
 } VglTrayIconPlaybackData;
 
 static void
-show_notification (VglTrayIcon *vti, LastfmTrack *track)
+show_notification                       (VglTrayIcon *vti,
+                                         LastfmTrack *track)
 {
         g_return_if_fail(VGL_IS_TRAY_ICON(vti) && track != NULL);
-        VglTrayIconPrivate *priv = VGL_TRAY_ICON_GET_PRIVATE (vti);
+        VglTrayIconPrivate *priv = vti->priv;
 
         GdkPixbuf *icon = NULL;
         gchar *notification_summary = NULL;
         gchar *notification_body = NULL;
         int i;
 
-        if (track->image_url != NULL) {
+        if (track->image_data != NULL) {
                 /* Set album image as icon if specified */
-                icon = get_album_cover_icon (track);
+                icon = get_pixbuf_from_image (track->image_data,
+                                              track->image_data_size,
+                                              NOTIFICATION_ICON_SIZE);
         }
 
         /* Set summary text (title) */
@@ -582,15 +542,16 @@ show_notification (VglTrayIcon *vti, LastfmTrack *track)
         g_free (notification_body);
 }
 
-static gpointer
-notify_playback_thread (VglTrayIconPlaybackData *d)
+static gboolean
+notify_playback_idle                    (gpointer userdata)
 {
-        g_return_val_if_fail(d != NULL, NULL);
+        VglTrayIconPlaybackData *d = userdata;
         VglTrayIconPrivate *priv;
 
-        gdk_threads_enter();
+        g_return_val_if_fail (d != NULL, FALSE);
+
         /* Set the now_playing private attribute and update panel */
-        priv = VGL_TRAY_ICON_GET_PRIVATE (d->vti);
+        priv = d->vti->priv;
         priv->now_playing = (d->track != NULL);
         ctxt_menu_update (d->vti);
 
@@ -611,32 +572,49 @@ notify_playback_thread (VglTrayIconPlaybackData *d)
                         show_notification (d->vti, d->track);
                 }
         } else {
-                gtk_status_icon_set_tooltip(priv->tray_icon, TOOLTIP_DEFAULT_STRING);
+                gtk_status_icon_set_tooltip (priv->tray_icon,
+                                             TOOLTIP_DEFAULT_STRING);
         }
-        gdk_threads_leave();
 
         /* Cleanup */
         g_object_unref(d->vti);
-        if (d->track != NULL) lastfm_track_unref(d->track);
+        if (d->track != NULL) vgl_object_unref(d->track);
         g_slice_free(VglTrayIconPlaybackData, d);
 
+        return FALSE;
+}
+
+static gpointer
+download_cover_thread                   (gpointer userdata)
+{
+        VglTrayIconPlaybackData *data = userdata;
+
+        lastfm_get_track_cover_image (data->track);
+        if (data->track->image_data == NULL) {
+                g_debug ("Error getting cover image");
+        }
+
+        gdk_threads_add_idle (notify_playback_idle, data);
         return NULL;
 }
 
-void
-vgl_tray_icon_notify_playback (VglTrayIcon *vti, LastfmTrack *track)
+static void
+vgl_tray_icon_notify_playback           (VglTrayIcon *vti,
+                                         LastfmTrack *track)
 {
         g_return_if_fail(VGL_IS_TRAY_ICON(vti));
         VglTrayIconPlaybackData *data;
+        gboolean dl_cover;
+
         data = g_slice_new(VglTrayIconPlaybackData);
         data->vti = g_object_ref(vti);
-        data->track = track ? lastfm_track_ref(track) : NULL;
-        g_thread_create((GThreadFunc)notify_playback_thread,data,FALSE,NULL);
-}
+        data->track = track ? vgl_object_ref(track) : NULL;
 
-void
-vgl_tray_icon_show_notifications (VglTrayIcon *vti, gboolean show_notifications)
-{
-        VglTrayIconPrivate *priv = VGL_TRAY_ICON_GET_PRIVATE (vti);
-        priv->show_notifications = show_notifications;
+        dl_cover = track && track->image_url && track->image_data == NULL;
+
+        if (vti->priv->show_notifications && dl_cover) {
+                g_thread_create (download_cover_thread, data, FALSE, NULL);
+        } else {
+                gdk_threads_add_idle (notify_playback_idle, data);
+        }
 }
