@@ -1,7 +1,9 @@
 /*
- * xmlrpc.c -- XMLRPC-based functions to tag artists, tracks, albums,
- *             loves and hates
- * Copyright (C) 2007 Alberto Garcia <agarcia@igalia.com>
+ * xmlrpc.c -- Legacy XMLRPC-based functions to tag artists, tracks,
+ *             albums, loves and hates
+ *
+ * Copyright (C) 2007-2009 Igalia, S.L.
+ * Authors: Alberto Garcia <agarcia@igalia.com>
  *
  * This file is part of Vagalume and is published under the GNU GPLv3
  * See the README file for more details.
@@ -15,7 +17,7 @@
 #include "http.h"
 #include "util.h"
 
-static const char *xmlrpc_url =
+static const char xmlrpc_url[] =
 "http://ws.audioscrobbler.com:80/1.0/rw/xmlrpc.php";
 
 /**
@@ -25,17 +27,13 @@ static const char *xmlrpc_url =
  * @return A new xmlNode *
  */
 static xmlNode *
-string_param(const char *str)
+string_param                            (const char *str)
 {
         g_return_val_if_fail(str != NULL, NULL);
         xmlNode *param = xmlNewNode(NULL, (xmlChar *) "param");
         xmlNode *value = xmlNewNode(NULL, (xmlChar *) "value");
-        xmlNode *string = xmlNewNode(NULL, (xmlChar *) "string");
-        xmlChar *encstr = xmlEncodeEntitiesReentrant(NULL, (xmlChar *)str);
         xmlAddChild(param, value);
-        xmlAddChild(value, string);
-        xmlNodeSetContent(string, encstr);
-        xmlFree(encstr);
+        xml_add_string (value, "string", str);
         return param;
 }
 
@@ -46,7 +44,7 @@ string_param(const char *str)
  * @return A new xmlNode *
  */
 static xmlNode *
-array_param(const GSList *params)
+array_param                             (const GSList *params)
 {
         const GSList *iter;
         xmlNode *param = xmlNewNode(NULL, (xmlChar *) "param");
@@ -58,14 +56,8 @@ array_param(const GSList *params)
         xmlAddChild(array, data);
         for (iter = params; iter != NULL; iter = g_slist_next(iter)) {
                 xmlNode *val = xmlNewNode(NULL, (xmlChar *) "value");
-                xmlNode *string = xmlNewNode(NULL, (xmlChar *) "string");
-                xmlChar *encstr;
-                encstr = xmlEncodeEntitiesReentrant(NULL,
-                                                    (xmlChar *) iter->data);
                 xmlAddChild(data, val);
-                xmlAddChild(val, string);
-                xmlNodeSetContent(string, encstr);
-                xmlFree(encstr);
+                xml_add_string (val, "string", (const char *) iter->data);
         }
         return param;
 }
@@ -81,14 +73,17 @@ array_param(const GSList *params)
  *         by the caller
  */
 static char *
-new_request(const char *user, const char *password, const char *method, ...)
+new_request                             (const char *user,
+                                         const char *password,
+                                         const char *method,
+                                         ...)
 {
         g_return_val_if_fail(user && password && method, NULL);
         va_list ap;
         char *retval;
         xmlChar *xmlbuff;
         int buffersize;
-        xmlDocPtr doc = xmlNewDoc((xmlChar *) "1.0");;
+        xmlDocPtr doc = xmlNewDoc((xmlChar *) "1.0");
         xmlNode *root = xmlNewNode(NULL, (xmlChar *) "methodCall");
         xmlNode *name = xmlNewNode(NULL, (xmlChar *) "methodName");
         xmlNode *params = xmlNewNode(NULL, (xmlChar *) "params");
@@ -128,14 +123,15 @@ new_request(const char *user, const char *password, const char *method, ...)
  * @return Whether the operation was successful or not
  */
 static gboolean
-xmlrpc_send_request(const char *request, const char *name)
+xmlrpc_send_request                     (const char *request,
+                                         const char *name)
 {
         g_return_val_if_fail(request != NULL, FALSE);
         gboolean retval = FALSE;
         GSList *headers = NULL;
         char *retbuf = NULL;
         headers = g_slist_append(headers, "Content-Type: text/xml");
-        http_post_buffer(xmlrpc_url, request, &retbuf, headers);
+        http_post_buffer (xmlrpc_url, request, &retbuf, NULL, headers);
 
         /* Check its return value */
         if (retbuf != NULL && g_strrstr(retbuf, "OK")) {
@@ -164,8 +160,11 @@ xmlrpc_send_request(const char *request, const char *name)
  * @return Whether the operation was successful or not
  */
 gboolean
-tag_track(const char *user, const char *password, const LastfmTrack *track,
-          request_type type, GSList *tags)
+tag_track                               (const char           *user,
+                                         const char           *password,
+                                         const LastfmTrack    *track,
+                                         LastfmTrackComponent  type,
+                                         GSList               *tags)
 {
         g_return_val_if_fail(user && password && track, FALSE);
         gboolean retval;
@@ -174,11 +173,11 @@ tag_track(const char *user, const char *password, const LastfmTrack *track,
         const char *method;
         param3 = array_param(tags);
         param4 = string_param("set");                  /* or use "append" */
-        if (type == REQUEST_ARTIST) {
+        if (type == LASTFM_TRACK_COMPONENT_ARTIST) {
                 method = "tagArtist";
                 param1 = string_param(track->artist);
                 param2 = NULL;
-        } else if (type == REQUEST_TRACK) {
+        } else if (type == LASTFM_TRACK_COMPONENT_TRACK) {
                 method = "tagTrack";
                 param1 = string_param(track->artist);
                 param2 = string_param(track->title);
@@ -200,34 +199,6 @@ tag_track(const char *user, const char *password, const LastfmTrack *track,
         return retval;
 }
 
-/**
- * Mark a track as loved or banned. This is the old protocol and will
- * be overriden by the Audioscrobbler Realtime Submission Protocol
- * v1.2 (see rsp_rscrobble()), but now it's still necessary
- *
- * @param user The user's Last.fm ID
- * @param password The user's password
- * @param track The track to mark as loved or banned
- * @param love TRUE to love, FALSE to ban
- * @return Whether the operation was successful or not
- */
-gboolean
-love_ban_track(const char *user, const char *password,
-               const LastfmTrack *track, gboolean love)
-{
-        g_return_val_if_fail(user && password && track, FALSE);
-        gboolean retval;
-        char *request;
-        xmlNode *artist, *title;
-        const char *method = love ? "loveTrack" : "banTrack";
-        artist = string_param(track->artist);
-        title = string_param(track->title);
-        request = new_request(user, password, method, artist, title, NULL);
-        retval = xmlrpc_send_request(request, method);
-        g_free(request);
-        return retval;
-}
-
 /**
  * Recommend a track to a user
  *
@@ -240,9 +211,12 @@ love_ban_track(const char *user, const char *password,
  * @return Whether the operation was successful or not
  */
 gboolean
-recommend_track(const char *user, const char *password,
-                const LastfmTrack *track, const char *text,
-                request_type type, const char *rcpt)
+recommend_track                         (const char           *user,
+                                         const char           *password,
+                                         const LastfmTrack    *track,
+                                         const char           *text,
+                                         LastfmTrackComponent  type,
+                                         const char           *rcpt)
 {
         g_return_val_if_fail(user && password && track && text && rcpt, FALSE);
         gboolean retval;
@@ -250,11 +224,11 @@ recommend_track(const char *user, const char *password,
         const char *method = "recommendItem";
         xmlNode *artist, *title, *recomm_type, *recomm_to;
         xmlNode *recomm_body, *language;
-        if (type == REQUEST_ARTIST) {
+        if (type == LASTFM_TRACK_COMPONENT_ARTIST) {
                 artist = string_param(track->artist);
                 title = string_param("");
                 recomm_type = string_param("artist");
-        } else if (type == REQUEST_TRACK) {
+        } else if (type == LASTFM_TRACK_COMPONENT_TRACK) {
                 artist = string_param(track->artist);
                 title = string_param(track->title);
                 recomm_type = string_param("track");
@@ -283,8 +257,9 @@ recommend_track(const char *user, const char *password,
  * @return Whether the operation was successful or not
  */
 gboolean
-add_to_playlist(const char *user, const char *password,
-                const LastfmTrack *track)
+add_to_playlist                         (const char        *user,
+                                         const char        *password,
+                                         const LastfmTrack *track)
 {
         g_return_val_if_fail(user && password && track, FALSE);
         gboolean retval;
